const recursive = require('recursive-readdir')
const db = require('./db')
const { camelCase } = require('lodash')
const camelcaseKeys = require('camelcase-keys')
const AJV = require('ajv')
const ajv = new AJV()

module.exports = async function init (fns) {
  const obj = {}

  let schemas = recursive(__dirname + '/jsonschema').then(files => {
    return files.map(f => require(f))
  })

  ajv.addSchema(await schemas)

  await fns.rows.forEach(f => {
    let schema = camelCase(f.schema)
    let name = camelCase(f.name)

    if (!obj[schema]) obj[schema] = {}

    let validate = ajv.getSchema(`https://terbo.org/schemas/${f.schema}/${f.name}.json`)
    let query = outerSelect(f.retset, subSelect(
      f.schema, f.name, payload
    ))
    obj[schema][name] = async function (payload) {
      if (!validate(payload)) {
        return Promise.reject({
          message: 'Invalid payload',
          details: validate.errors
        })
      }

      try {
        let result = await db.query(query)
        return camelcaseKeys(result.rows[0].result, { deep: true })
      } catch (e) {
        return Promise.reject(e)
      }
    }
  })

  return obj
}

function subSelect (schema, name, payload) {
  payload = `$$${JSON.stringify(payload)}$$::json`

  return `SELECT * FROM ${schema}.${name}(${payload})`
}

function outerSelect (retset, subSelect) {
  let columnList = `json_strip_nulls(${
    retset ? 'json_agg' : 'to_json'
  }(r)) AS result`

  return `SELECT ${columnList} FROM (${subSelect}) r;`
}
