CREATE OR REPLACE FUNCTION core.create_organisation (json, OUT result core.organisation_v) AS
$$
BEGIN
  WITH payload AS (
    SELECT
      p.*
    FROM json_to_record($1) AS p (
      name          text,
      email         text,
      uri           text,
      trading_name  text,
      data          jsonb,
      url           text,
      industry_code text
    )
  ), new_party AS (
    INSERT INTO core.party (
      name,
      kind,
      data,
      uri
    )
    SELECT
      p.name,
      'ORGANISATION',
      p.data,
      p.uri
    FROM payload p 
    RETURNING *
  ),
  tsv AS (
    INSERT INTO core.search (
      object_uuid,
      tsv
    )
    SELECT
      p.party_uuid,
      setweight(to_tsvector('simple', COALESCE(p.name, '')), 'B') ||
        setweight(to_tsvector('simple', COALESCE(p.uri, '')), 'A') ||
        setweight(to_tsvector('simple', COALESCE(p.email, '')), 'A')
    FROM new_party p
  ),
  new_organisation AS (
    INSERT INTO core.organisation (
      party_uuid,
      trading_name,
      url,
      industry_code
    )
    SELECT
      (SELECT party_uuid FROM new_party),
      p.trading_name,
      p.url,
      p.industry_code
    FROM payload p
    RETURNING *
  )
  SELECT INTO result
    p.*,
    org.trading_name,
    org.url,
    org.industry_code
  FROM new_party p
  LEFT JOIN new_organisation org
    USING (party_uuid);
END
$$
LANGUAGE 'plpgsql' SECURITY DEFINER;
