CREATE OR REPLACE FUNCTION core.create_person (json, OUT result core.person_v) AS
$$
BEGIN
  WITH payload AS (
    SELECT
      p.*
    FROM json_to_record($1) AS p (
      name            text,
      email           text,
      mobile          text,
      phone           text,
      timezone_abbrev text,
      language_code   text
    )
  ), new_party AS (
    INSERT INTO core.party (
      name,
      email,
      kind,
      timezone_abbrev,
      language_code
    )
    SELECT
      p.name,
      p.email,
      'PERSON',
      p.timezone_abbrev,
      p.language_code
    FROM payload p 
    RETURNING *
  ), tsv AS (
    INSERT INTO core.search (
      object_uuid,
      tsv
    )
    SELECT
      p.party_uuid,
      setweight(to_tsvector('simple', p.name), 'B') ||
        setweight(to_tsvector('simple', p.email), 'A')
    FROM new_party p
  ), new_person AS (
    INSERT INTO core.person (
      party_uuid,
      mobile,
      phone
    )
    SELECT
      (SELECT party_uuid FROM new_party),
      p.mobile,
      p.phone
    FROM payload p
    RETURNING *
  )
  SELECT INTO result
    p.*,
    prsn.mobile,
    prsn.phone
  FROM new_party p
  LEFT JOIN new_person prsn
    USING (party_uuid);
END
$$
LANGUAGE 'plpgsql' SECURITY DEFINER;
