CREATE OR REPLACE FUNCTION core.create_relationship (json, OUT result json)
LANGUAGE 'plpgsql' AS
$$
BEGIN
  WITH new_relationship AS (
    INSERT INTO core.party_relationship (
      relationship_id,
      first_party_uuid,
      second_party_uuid
    )
    SELECT
      p.relationship_id,
      first_party_uuid,
      second_party_uuid
    FROM json_to_record($1) AS p (
      relationship_id   integer,
      first_party_uuid  uuid,
      second_party_uuid uuid
    )
    RETURNING *
  )
  SELECT INTO result
    json_strip_nulls(to_json(r))
  FROM (
    SELECT
      nr.*,
      rv.name,
      rv.description,
      p1.name AS first_party_name,
      p2.name AS second_party_name
    FROM new_relationship nr
    LEFT JOIN core.relationship_v rv
      USING (relationship_id)
    LEFT JOIN core.party p1
      ON p1.party_uuid = nr.first_party_uuid
    LEFT JOIN core.party p2
      ON p2.party_uuid = nr.second_party_uuid
  ) r;
END
$$;