/**
 * Return all the functions that are not in system catalogs or public and where
 * the first argument is of type json
 */
CREATE OR REPLACE FUNCTION core.map_functions () 
RETURNS TABLE (
  name   name,
  schema name,
  retset boolean
) AS
$$
BEGIN
  RETURN QUERY
  SELECT
    proc.proname,
    ns.nspname,
    proc.proretset AS retset
  FROM pg_catalog.pg_proc proc
  INNER JOIN pg_catalog.pg_namespace ns
    ON ns.oid = proc.pronamespace
  INNER JOIN pg_catalog.pg_type arg
    ON arg.oid = proc.proargtypes[0]
      AND arg.typname = 'json'
  WHERE ns.nspname NOT IN (
    'pg_catalog',
    'information_schema',
    'public'
  );
END
$$
LANGUAGE 'plpgsql' SECURITY DEFINER;
