/*
Create the necessary purchase orders for a sales.order.
*/

CREATE OR REPLACE FUNCTION pcm.create_purchase_order (json, OUT result json) AS
$$
BEGIN
  WITH payload AS (
    SELECT
      p.seller_uuid,
      p.buyer_uuid
    FROM json_to_record($1) AS p (
      seller_uuid uuid,
      buyer_uuid  uuid
    )
  ), purchase_order AS (
    INSERT INTO pcm.purchase_order (
      seller_uuid,
      buyer_uuid
    )
    SELECT
      *
    FROM payload
    RETURNING *
  )
  SELECT json_strip_nulls(to_json(r)) INTO result
  FROM (
    SELECT
      po.purchase_order_uuid,
      po.seller_uuid,
      po.buyer_uuid,
      obj.name AS object_name,
      d.document_num,
      d.document_status,
      s.name AS seller_name,
      b.name AS buyer_name
    FROM purchase_order po
    LEFT JOIN core.object obj
      ON obj.object_uuid = po.purchase_order_uuid
    LEFT JOIN core.document_v d
      ON d.document_uuid = po.purchase_order_uuid
    LEFT JOIN core.party s
      ON s.party_uuid = po.seller_uuid
    LEFT JOIN core.party b
      ON b.party_uuid = po.buyer_uuid
  ) r;
END
$$
LANGUAGE 'plpgsql' SECURITY DEFINER;
