CREATE OR REPLACE FUNCTION pcm.purchase_order_document (
  json DEFAULT NULL,
  purchase_order uuid DEFAULT NULL,
  OUT result json
) LANGUAGE 'plpgsql' AS
$$
BEGIN
  SELECT json_strip_nulls(to_json(r)) INTO result
  FROM (
    SELECT
      po.purchase_order_uuid,
      po.document_num,
      po.reference_num,
      po.seller_uuid,
      po.buyer_uuid,
      po.document_status,
      to_char(po.created, core.setting('default_date_format')) AS created,
      pcm.details(purchase_order_uuid => po.purchase_order_uuid) AS details,
      to_json(s) AS seller,
      to_json(b) AS buyer,
      to_json(sc) AS seller_contact,
      to_json(bc) AS buyer_contact,
      to_json(da) AS delivery_address
    FROM pcm.purchase_order_v po
    LEFT JOIN core.party s
      ON s.party_uuid = po.seller_uuid
    LEFT JOIN core.party b
      ON b.party_uuid = po.buyer_uuid
    LEFT JOIN core.party sc
      ON sc.party_uuid = po.seller_contact_uuid
    LEFT JOIN core.party bc
      ON bc.party_uuid = po.buyer_contact_uuid
    LEFT JOIN core.address_v da
      ON da.address_uuid = po.delivery_address_uuid
    WHERE po.purchase_order_uuid = ($1->>'purchase_order_uuid')::uuid
      OR po.purchase_order_uuid = $2
  ) r;
END
$$;
