CREATE OR REPLACE FUNCTION pcm.update_detail (json, OUT result json) AS
$$
BEGIN
  EXECUTE (
    SELECT
      format('UPDATE pcm.detail SET (%s) = (%s) WHERE detail_uuid = ''%s''', c.column, c.value, c.detail_uuid)
    FROM (
      SELECT
        string_agg(q.column, ', ') AS column,
        string_agg(q.value, ', ') AS value,
        ($1->>'detail_uuid')::uuid AS detail_uuid
      FROM (
        SELECT
          p.key AS column,
          CASE
            -- check if it's a number
            WHEN p.value ~ '^\d+(.\d+)?$' THEN
              p.value
            WHEN p.value IS NULL THEN
              'NULL'
            ELSE quote_literal(p.value)
          END AS value
        FROM json_each_text($1) p
        WHERE p.key != 'detail_uuid'
      ) q
    ) c
  );

  SELECT INTO result
    json_strip_nulls(to_json(r))
  FROM (
    SELECT
      d.*
    FROM pcm.detail d
    WHERE d.detail_uuid = ($1->>'detail_uuid')::uuid
  ) r;
END
$$
LANGUAGE 'plpgsql' SECURITY DEFINER;
