CREATE OR REPLACE FUNCTION pcm.update_purchase_order (json, OUT result json) AS
$$
BEGIN    
  EXECUTE (
    SELECT
      CASE
        WHEN c.column IS NOT NULL THEN
          format('UPDATE pcm.purchase_order SET (%s) = (%s) WHERE purchase_order_uuid = ''%s''', c.column, c.value, ($1->>'purchase_order_uuid'))
        ELSE ''
      END
    FROM (
      SELECT
        string_agg(q.column, ', ') AS column,
        string_agg(q.value, ', ') AS value
      FROM (
        SELECT
          p.key AS column,
          CASE
            -- check if it's a number
            WHEN p.value ~ '^\d+(.\d+)?$' THEN
              p.value
            WHEN p.value IS NULL THEN
              'NULL'
            ELSE quote_literal(p.value)
          END AS value
        FROM json_each_text($1) p
        WHERE p.key != 'purchase_order_uuid'
          AND p.key IN (
            SELECT
              col.column_name
            FROM information_schema.columns col
            WHERE col.table_schema = 'pcm' AND col.table_name = 'purchase_order'
          )
      ) q
    ) c
  );

  EXECUTE (
    SELECT
      CASE
        WHEN c.column IS NOT NULL THEN
          format('UPDATE core.document SET (%s) = (%s) WHERE document_uuid = ''%s''', c.column, c.value, ($1->>'purchase_order_uuid'))
        ELSE ''
      END
    FROM (
      SELECT
        string_agg(q.column, ', ') AS column,
        string_agg(q.value, ', ') AS value
      FROM (
        SELECT
          p.key AS column,
          CASE
            -- check if it's a number
            WHEN p.value ~ '^\d+(.\d+)?$' THEN
              p.value
            WHEN p.value IS NULL THEN
              'NULL'
            ELSE quote_literal(p.value)
          END AS value
        FROM json_each_text($1) p
        WHERE p.key IN (
            SELECT
              col.column_name
            FROM information_schema.columns col
            WHERE col.table_schema = 'core' AND col.table_name = 'document'
          )
      ) q
    ) c
  );

  SELECT pcm.purchase_order(purchase_order => ($1->>'purchase_order_uuid')::uuid) INTO result;
END
$$
LANGUAGE 'plpgsql' SECURITY DEFINER;
