-- required: core
-- required: product

CREATE SCHEMA pcm;

CREATE TABLE pcm.purchase (
  purchase_uuid uuid REFERENCES core.object (object_uuid) PRIMARY KEY
);

CREATE TABLE pcm.purchase_order (
  purchase_order_uuid   uuid REFERENCES core.document PRIMARY KEY,
  purchase_uuid         uuid REFERENCES pcm.purchase (purchase_uuid),
  reference_num         text,
  reference_object      uuid REFERENCES core.object (object_uuid) ON DELETE SET NULL,
  seller_uuid           uuid REFERENCES core.party (party_uuid) NOT NULL,
  buyer_uuid            uuid REFERENCES core.party (party_uuid) NOT NULL,
  seller_contact_uuid   uuid REFERENCES core.party (party_uuid) ON DELETE SET NULL,
  buyer_contact_uuid    uuid REFERENCES core.party (party_uuid) ON DELETE SET NULL,
  delivery_address_uuid uuid REFERENCES core.address (address_uuid) ON DELETE SET NULL,
  data                  jsonb
);

CREATE TABLE pcm.detail (
  detail_uuid         uuid REFERENCES core.object PRIMARY KEY,
  purchase_uuid       uuid REFERENCES pcm.purchase (purchase_uuid),
  purchase_order_uuid uuid REFERENCES pcm.purchase_order (purchase_order_uuid) ON DELETE CASCADE,
  product_uuid        uuid REFERENCES prd.product (product_uuid) ON DELETE RESTRICT,
  quantity            numeric(10,3),
  price               numeric(10,2),
  created             timestamptz DEFAULT CURRENT_TIMESTAMP,
  modified            timestamptz DEFAULT CURRENT_TIMESTAMP
);

CREATE VIEW pcm.purchase_order_v AS
  SELECT
    po.purchase_order_uuid,
    po.reference_num,
    po.seller_uuid,
    po.buyer_uuid,
    po.seller_contact_uuid,
    po.buyer_contact_uuid,
    po.delivery_address_uuid,
    obj.name AS object_name,
    obj.created,
    d.document_num,
    d.document_status
  FROM pcm.purchase_order po
  LEFT JOIN core.object obj
    ON obj.object_uuid = po.purchase_order_uuid
  LEFT JOIN core.document_v d
    ON d.document_uuid = po.purchase_order_uuid;

--
-- Triggers
--

CREATE OR REPLACE FUNCTION pcm.insert_po_trigger () RETURNS TRIGGER AS
$$
BEGIN
  WITH core_object AS (
    INSERT INTO core.object (
      name,
      schema
    ) VALUES (
      TG_TABLE_NAME,
      TG_TABLE_SCHEMA
    )
    RETURNING *
  )
  INSERT INTO core.document (
    document_uuid,
    status_id
  )
  SELECT
    co.object_uuid,
    1 -- status_id of default document_status: 'draft'
  FROM core_object co
  RETURNING document_uuid INTO NEW.purchase_order_uuid;

  RETURN NEW;
END
$$
LANGUAGE 'plpgsql';

CREATE TRIGGER pcm_purchase_order_insert BEFORE INSERT ON pcm.purchase_order
FOR EACH ROW EXECUTE PROCEDURE pcm.insert_po_trigger();

--
-- Detail
--
CREATE OR REPLACE FUNCTION pcm.insert_detail_trigger () RETURNS TRIGGER AS
$$
BEGIN
  INSERT INTO core.object (
    name,
    schema
  ) VALUES (
    TG_TABLE_NAME,
    TG_TABLE_SCHEMA
  )
  RETURNING object_uuid INTO NEW.detail_uuid;

  RETURN NEW;
END
$$
LANGUAGE 'plpgsql';

CREATE TRIGGER pcm_detail_insert BEFORE INSERT ON pcm.detail
FOR EACH ROW EXECUTE PROCEDURE pcm.insert_detail_trigger();
