/**
 * Return the products and quantities from an assembly.
 * Only parts that have a product_uuid and a quantity are returned
 */
CREATE OR REPLACE FUNCTION prd.assembly_quantities (uuid)
RETURNS TABLE (
  product_uuid uuid,
  quantity     numeric(10,3)
) AS
$$
BEGIN
  RETURN QUERY
  WITH RECURSIVE parts AS (
    SELECT
      p.part_uuid,
      p.parent_uuid,
      p.product_uuid,
      p.quantity
    FROM prd.part p
    WHERE p.parent_uuid = $1
    
    UNION ALL
    
    SELECT
      p.part_uuid,
      p.parent_uuid,
      p.product_uuid,
      p.quantity
    FROM parts
    INNER JOIN prd.part p ON p.parent_uuid = parts.part_uuid
  )
  SELECT
    p.product_uuid,
    p.quantity
  FROM parts p
  WHERE p.product_uuid IS NOT NULL AND p.quantity IS NOT NULL;
END
$$
LANGUAGE 'plpgsql' SECURITY DEFINER;
