CREATE OR REPLACE FUNCTION prd.children (json DEFAULT NULL, parent uuid DEFAULT NULL, OUT result json)
LANGUAGE 'plpgsql' AS $$
BEGIN
  WITH children AS (
    SELECT
      json_strip_nulls(to_json(r)) AS obj
    FROM (
      SELECT
        p.root_uuid,
        p.product_uuid,
        p.part_uuid,
        p.name,
        p.quantity,
        pr.name AS product_name,
        pr.kind AS product_kind,
        prd.children(parent => p.part_uuid) AS children
      FROM prd.part p
      LEFT JOIN prd.product_v pr
        USING (product_uuid)
      WHERE p.parent_uuid = COALESCE($2, ($1->>'product_uuid')::uuid)
        AND p.quantity IS NOT NULL
    ) r

    UNION ALL

    SELECT
      json_strip_nulls(to_json(r)) AS obj
    FROM (
      SELECT
        p.*,
        pr.name AS product_name,
        prd.children(parent => p.part_uuid) AS children
      FROM prd.part p
      LEFT JOIN prd.product_v pr
        USING (product_uuid)
      WHERE p.parent_uuid = COALESCE($2, ($1->>'product_uuid')::uuid)
        AND p.quantity IS NULL
    ) r
  )
  SELECT INTO result
    json_agg(c.obj)
  FROM children c;
END
$$;
