/**
 * Calculate the cost of a product.
 * A product could be a composite, so if no cost is explicitly assigned and the
 * product is a composite, return the composite cost.
 */
CREATE OR REPLACE FUNCTION prd.cost (uuid) RETURNS prd.cost_t AS
$$
DECLARE
  _product prd.product_v;
  result  prd.cost_t;
BEGIN
  SELECT
    p.*
  INTO
    _product
  FROM prd.product_v p
  WHERE p.product_uuid = $1;

  SELECT DISTINCT ON (c.product_uuid)
    c.cost_uuid,
    $1,
    c.amount,
    CASE
      WHEN c.amount IS NOT NULL THEN
        TRUE
      ELSE NULL
    END,
    c.created,
    c.end_at
  INTO
    result.cost_uuid,
    result.product_uuid,
    result.amount,
    result.amount_is_set,
    result.created,
    result.end_at
  FROM prd.cost c
  WHERE c.product_uuid = $1
  ORDER BY c.product_uuid, c.created DESC;

  IF result IS NULL AND _product.is_composite IS TRUE THEN
    SELECT
      c.cost_uuid,
      c.product_uuid,
      c.amount,
      c.created,
      c.end_at,
      FALSE
    INTO
      result.cost_uuid,
      result.product_uuid,
      result.amount,
      result.created,
      result.end_at,
      result.amount_is_set
    FROM prd.composite_cost($1) c;
  ELSIF result IS NULL AND _product.part_uuid IS NOT NULL THEN
    SELECT
      $1,
      sum((q.quantity * c.amount)::numeric(10,2))
    INTO
      result.product_uuid,
      result.amount
    FROM prd.assembly_quantities($1) q
    LEFT JOIN prd.cost(q.product_uuid) c
      ON c.product_uuid = q.product_uuid;
  END IF;

  RETURN result;
END
$$
LANGUAGE 'plpgsql';

CREATE OR REPLACE FUNCTION prd.cost (json, OUT result json) AS
$$
BEGIN
  SELECT
    json_strip_nulls(to_json(r))
  INTO
    result
  FROM prd.cost(($1->>'product_uuid')::uuid) r;
END
$$
LANGUAGE 'plpgsql';
