CREATE OR REPLACE FUNCTION prj.create_deliverable (json, OUT result json) AS
$$
DECLARE
  _sequence_num integer;
  _item_uuid uuid;
BEGIN
  SELECT INTO _sequence_num
    COALESCE(($1->>'sequence_num')::integer, max(d.sequence_num) + 1, 1)
  FROM prj.deliverable d
  WHERE d.job_uuid = ($1->>'job_uuid')::uuid;
  
  IF $1->>'product_uuid' IS NOT NULL AND $1->>'name' IS NOT NULL THEN
    _item_uuid = scm.create_item(($1->>'product_uuid')::uuid, $1->>'name');
  ELSIF $1->>'item_uuid' IS NOT NULL THEN
    _item_uuid = ($1->>'item_uuid')::uuid;
  ELSE
    INSERT INTO scm.item (
      name
    ) VALUES (
      $1->>'name'
    )
    RETURNING item_uuid INTO _item_uuid;
  END IF;

  WITH deliverable AS (
    INSERT INTO prj.deliverable (
      job_uuid,
      item_uuid,
      sequence_num
    ) VALUES (
      ($1->>'job_uuid')::uuid,
      _item_uuid,
      _sequence_num
    )
    RETURNING *
  )
  SELECT json_strip_nulls(to_json(r)) INTO result
  FROM (
    SELECT
      d.deliverable_uuid,
      $1->>'name' AS name,
      _item_uuid
    FROM deliverable d
  ) r;
END
$$
LANGUAGE 'plpgsql';
