/**
 * Enqueue all Items in a project to the task queue.
 */
CREATE OR REPLACE FUNCTION prj.enqueue_items (
  json DEFAULT NULL,
  project_uuid uuid DEFAULT NULL,
  OUT result json
) AS
$$
BEGIN
  INSERT INTO scm.batch (
    batch_uuid,
    data
  ) VALUES (
    COALESCE(($1->>'project_uuid')::uuid, $2),
    format('{"project_uuid":"%s"}', COALESCE($1->>'project_uuid', $2::text))::jsonb
  )
  ON CONFLICT DO NOTHING;

  WITH items AS (
    SELECT
      i.item_uuid
    FROM prj.flatten_job(
      (
        SELECT
          pj.job_uuid
        FROM prj.project_job pj
        WHERE pj.project_uuid = COALESCE(($1->>'project_uuid')::uuid, $2)
      )
    ) j
    INNER JOIN prj.deliverable d
      ON d.job_uuid = j.job_uuid
    INNER JOIN scm.item i
      ON i.item_uuid = d.item_uuid AND i.product_uuid IS NOT NULL
  ),
  tasks AS (
    SELECT
      count(*) AS count
    FROM items i
    LEFT JOIN scm.enqueue_item(
      i.item_uuid,
      COALESCE(($1->>'project_uuid')::uuid, $2)
    ) e
      ON e.item_uuid = i.item_uuid
  )
  SELECT INTO result
    to_json(t)
  FROM tasks t;
END
$$
LANGUAGE 'plpgsql';
