/**
 * Job progress is determined by checking the state of all the terminals in a
 * job. Terminals can be jobs, deliverables or tasks of deliverables.
 */
CREATE OR REPLACE FUNCTION prj.job_progress (json, OUT result json) AS
$$
BEGIN
  SELECT json_strip_nulls(to_json(r)) INTO result
  FROM (
    SELECT
      j.job_uuid,
      j.dependant_uuid,
      j.name,
      j.short_desc,
      -- prg.finished_task_count AS "finishedTaskCount",
      -- prg.total_task_count AS "totalTaskCount",
      -- floor(prg.finished_task_count::numeric / prg.total_task_count::numeric * 100) AS "percentComplete"
      (
        SELECT
          array_agg(r)
        FROM (
          SELECT
            d.deliverable_uuid,
            COALESCE(prg.percent_complete, 0) AS percent_complete
          FROM prj.deliverable d
          LEFT JOIN scm.item_progress(d.item_uuid) prg
            ON prg.item_uuid = d.item_uuid
          WHERE d.job_uuid = j.job_uuid
        ) r
      ) AS deliverables
    FROM prj.job j
    -- Calculate job progress
    -- LEFT JOIN LATERAL (
    --   SELECT
    --     j.job_id,
    --     sum(prg.finished_task_count) AS finished_task_count,
    --     sum(prg.total_task_count) AS total_task_count
    --   FROM prj.flatten_job(j.job_id) fj
    --   INNER JOIN prj.deliverable d
    --     ON d.job_id = fj.job_id
    --   INNER JOIN scm.item_progress(d.item_uuid) prg
    --     ON prg.item_uuid = d.item_uuid
    -- ) prg ON prg.job_id = j.job_id
    WHERE j.job_uuid = ($1->>'job_uuid')::uuid
  ) r;
END
$$
LANGUAGE 'plpgsql' SECURITY DEFINER;
