CREATE OR REPLACE FUNCTION sales.list_orders (json DEFAULT NULL, OUT result json) AS
$$
BEGIN
  IF $1->>'search' IS NULL THEN
    SELECT json_strip_nulls(json_agg(r)) INTO result
    FROM (
      SELECT
        o.order_uuid,
        d.document_num,
        o.order_status,
        o.nickname,
        o.short_desc,
        o.customer_uuid,
        cst.name AS customer_name,
        o.created,
        o.start_date
      FROM sales.order o
      LEFT JOIN core.document d
        ON d.document_uuid = o.order_uuid
      LEFT JOIN core.party cst
        ON cst.party_uuid = o.customer_uuid
      ORDER BY o.created DESC
      LIMIT COALESCE(($1->>'limit')::integer, 20) OFFSET COALESCE(($1->>'offset')::integer, 0)
    ) r;
  ELSE
    SELECT json_strip_nulls(json_agg(r)) INTO result
    FROM (
      SELECT
        o.order_uuid,
        d.document_num,
        o.status,
        o.nickname,
        o.short_desc,
        o.customer_uuid,
        cst.name AS customer_name,
        o.created
      FROM sales.order o
      LEFT JOIN core.document d
        ON d.document_uuid = o.order_uuid
      LEFT JOIN core.party cst
        ON cst.party_uuid = o.customer_uuid
      WHERE o.tsv @@ to_tsquery(($1->>'search') || ':*')
      ORDER BY o.created DESC
    ) r;
  END IF;
END
$$
LANGUAGE 'plpgsql' SECURITY DEFINER;
