CREATE OR REPLACE FUNCTION sales.product (uuid, OUT result sales.product_v)
LANGUAGE 'plpgsql' AS $$
DECLARE
  _product prd.product_v;
BEGIN
  SELECT INTO result
    p.*
  FROM sales.product_v p
  WHERE p.product_uuid = $1;

  IF result.price IS NOT NULL OR result.amount_payable IS NOT NULL THEN
    RETURN;
  END IF;
  
  SELECT INTO _product
    p.*
  FROM prd.product_v p
  WHERE p.product_uuid = $1;

  -- A price has not been explicity set on the product. We need to figure it out
  -- If there is no cost, try to get a cost amount
  IF result.cost IS NULL THEN
    SELECT
      c.amount
    INTO
      result.cost
    FROM prd.cost($1) c;
  END IF;
  
  -- If there is a cost available, we can try to determine the price if there
  -- is a margin or markup
  IF result.cost IS NOT NULL THEN
    IF result.margin IS NOT NULL THEN
      result.price = (result.cost * (1 / (1 - result.margin)))::numeric(10,2);
    ELSIF result.markup IS NOT NULL THEN
      result.price = (result.cost * (1 + result.markup))::numeric(10,2);
    ELSE
      IF _product.part_uuid IS NOT NULL THEN
        SELECT
          sum((q.quantity * p.price)::numeric(10,2))
        INTO
          result.price
        FROM prd.assembly_quantities(_product.product_uuid) q
        LEFT JOIN sales.product(q.product_uuid) p
          ON p.product_uuid = q.product_uuid;
      ELSIF _product.is_composite IS TRUE THEN
        SELECT
          sum((q.quantity * p.price)::numeric(10,2))
        INTO
          result.price
        FROM prd.composite_quantities(_product.product_uuid) q
        LEFT JOIN sales.product(q.product_uuid) p
          ON p.product_uuid = q.product_uuid;
      END IF;
    END IF;
    result.margin = (result.price - result.cost) / result.price;
  ELSE
    RAISE NOTICE 'this i s else';
  END IF;
  RETURN;
END
$$;

CREATE OR REPLACE FUNCTION sales.product (json, OUT result json) 
LANGUAGE 'plpgsql' AS
$$
BEGIN
  SELECT INTO result
    json_strip_nulls(to_json(r))
  FROM (
    SELECT
      s.*,
      p.*
    FROM sales.product(($1->>'product_uuid')::uuid) s
    LEFT JOIN prd.product_v p
      USING (product_uuid)
    WHERE p.product_uuid = ($1->>'product_uuid')::uuid
  ) r;
END
$$;
