CREATE OR REPLACE FUNCTION sales.quote_document (json, OUT result json) AS
$$
BEGIN
  WITH quote AS (
    SELECT
      q.*,
      d.document_num,
      (
        SELECT
          name
        FROM core.document_status ds
        WHERE ds.status_id = d.status_id
      ) AS approval_status
    FROM sales.quote q
    LEFT JOIN core.document d
      ON d.document_uuid = q.quote_uuid
    WHERE q.quote_uuid = ($1->>'quote_uuid')::uuid
  ),
  _order AS (
    SELECT
      o.*
    FROM quote q
    LEFT JOIN sales.order o
      ON o.order_uuid = q.order_uuid
  ),
  details AS (
    SELECT
      li.*,
      (COALESCE(li.quantity, 1.000) * li.price)::numeric(10,2) AS _total
    FROM _order o
    LEFT JOIN sales.line_items(_order_uuid => o.order_uuid) li
      ON li.order_uuid = o.order_uuid
  ),
  customer AS (
    SELECT
      cv.customer_uuid,
      cv.name,
      cv.kind,
      ad.addr1,
      ad.addr2,
      ad.locality,
      ad.subdivision,
      ad.country,
      ad.code
    FROM sales.customer_v cv
    LEFT JOIN core.address_v ad
      ON ad.address_uuid = cv.address_uuid
    WHERE cv.customer_uuid = (SELECT customer_uuid FROM _order)
  ), contact AS (
    SELECT
      p.name,
      pe.email
    FROM core.party p
    LEFT JOIN core.person pe
      USING (party_uuid)
    WHERE p.party_uuid = (SELECT contact_uuid FROM _order)
  )
  SELECT INTO result
    json_strip_nulls(to_json(r))
  FROM (
    SELECT
      q.*,
      to_char(q.expiry_date, core.setting('default_date_format')) AS expiry_date,
      (
        SELECT c FROM customer c
      ) AS customer,
      (
        SELECT c FROM contact c
      ) AS contact,
      (
        SELECT array_agg(d) FROM details d
      ) AS details,
      (
        SELECT
          sum(d._total)::numeric(10,2)
        FROM details d
      ) AS subtotal,
      (
        SELECT
          (sum(d._total) * 1.1)::numeric(10,2) -- TODO do taxes properly
        FROM details d
      ) AS amount_payable,
      (
        SELECT
          ((sum(d._total) * 1.1) - sum(d._total))::numeric(10,2)
        FROM details d
      ) AS tax_payable
    FROM quote q
  ) r;
END
$$
LANGUAGE 'plpgsql' SECURITY DEFINER;
