CREATE OR REPLACE FUNCTION scm.children (json DEFAULT NULL, parent uuid DEFAULT NULL, OUT result json)
LANGUAGE 'plpgsql' AS $$
BEGIN
  WITH children AS (
    SELECT
      json_strip_nulls(to_json(r)) AS obj
    FROM (
      SELECT
        c.*,
        p.name,
        p.kind AS product_kind,
        p.uom_abbr,
        p.name AS product_name
      FROM scm.component c
      LEFT JOIN prd.product_v p
        USING (product_uuid)
      WHERE c.item_uuid = COALESCE($2, ($1->>'item_uuid')::uuid)
    ) r

    UNION ALL

    SELECT
      json_strip_nulls(to_json(r)) AS obj
    FROM (
      SELECT
        p.*,
        i.name,
        pr.name AS product_name,
        scm.children(parent => p.item_uuid) AS children
      FROM scm.part p
      LEFT JOIN scm.item i
        USING (item_uuid)
      LEFT JOIN prd.product_v pr
        ON pr.product_uuid = i.product_uuid
      WHERE p.parent_uuid = COALESCE($2, ($1->>'item_uuid')::uuid)
    ) r
  )
  SELECT INTO result
    json_agg(c.obj)
  FROM children c;
END
$$;
