CREATE OR REPLACE FUNCTION scm.create_component (json, parent_uuid uuid DEFAULT NULL, OUT result json) AS
$$
DECLARE
  _i record;
BEGIN
  WITH payload AS (
    SELECT
      p.*
    FROM json_to_record($1) AS p (
      item_uuid     uuid,
      product_uuid  uuid,
      quantity      numeric(10,3),
      data          jsonb
    )
  ), new_component AS (
    INSERT INTO scm.component (
      item_uuid,
      product_uuid,
      quantity,
      data
    )
    SELECT
      coalesce($2, p.item_uuid),
      product_uuid,
      quantity,
      data
    FROM payload p
    RETURNING *
  )
  SELECT json_strip_nulls(to_json(r)) INTO result
  FROM (
    SELECT
      c.component_uuid,
      c.item_uuid,
      c.product_uuid,
      c.quantity,
      c.data,
      p.name
    FROM new_component c
    LEFT JOIN prd.product_v p
      USING (product_uuid)
  ) r;

  -- Create components from children
  FOR _i IN SELECT * FROM json_array_elements($1->'components')
  LOOP
    PERFORM scm.create_component(_i.value, (result->>'item_uuid')::uuid);
  END LOOP;
END
$$
LANGUAGE 'plpgsql' SECURITY DEFINER;
