CREATE OR REPLACE FUNCTION scm.create_item (json, OUT result json) AS
$$
DECLARE
  new_item_uuid uuid;
  i RECORD;
BEGIN
  INSERT INTO scm.item (
    product_uuid,
    name
  )
  SELECT
    p.product_uuid,
    p.name
  FROM json_to_record($1) AS p (
    product_uuid    uuid,
    name            text
  )
  RETURNING item_uuid INTO new_item_uuid;

  IF  json_typeof($1->'attributes') = 'array' THEN
    FOR i IN SELECT
               *
             FROM json_to_recordset($1->'attributes') AS a (
               name text, value text
             ) LOOP

      INSERT INTO scm.attribute (
        item_uuid,
        name,
        value
      ) VALUES (
        new_item_uuid,
        i.name,
        i.value
      );
    END LOOP;
  END IF;

  SELECT json_strip_nulls(to_json(scm.item(new_item_uuid))) INTO result;
END
$$
LANGUAGE 'plpgsql' SECURITY DEFINER;

/**
 * Create an item based on a product
 */
CREATE OR REPLACE FUNCTION scm.create_item (product_uuid uuid, name text DEFAULT NULL, OUT result uuid) AS
$$
DECLARE
  i RECORD;
BEGIN
  INSERT INTO scm.item (
    product_uuid,
    name
  ) VALUES (
    $1,
    $2
  )
  RETURNING item_uuid INTO result;

  INSERT INTO scm.attribute (
    item_uuid,
    name,
    value
  )
  SELECT
    result,
    a.name,
    a.value
  FROM prd.product_attribute a
  WHERE a.product_uuid = $1;

  INSERT INTO scm.part (
    parent_uuid,
    item_uuid
  )
  SELECT
    result,
    scm.create_item(p.product_uuid, p.name)
  FROM prd.part p
  WHERE p.parent_uuid = $1;
END
$$
LANGUAGE 'plpgsql' SECURITY DEFINER;
