/**
Return tasks in the queue that are ready. Do not return tasks where the
corresponding Item's children's tasks have not been finished.
*/

CREATE OR REPLACE FUNCTION scm.task_queue (json, OUT result json) AS
$$
BEGIN
  WITH task_queue AS (
    SELECT
      q.*
    FROM scm.task_queue_v q
    WHERE q.finished_at IS NULL
      AND q.work_center_uuid = ($1->>'work_center_uuid')::uuid
  )
  SELECT json_strip_nulls(json_agg(r)) INTO result
  FROM (
    SELECT
      q.task_uuid,
      q.item_uuid,
      q.route_uuid,
      q.sequence_num,
      q.batch_uuid,
      i.name AS item_name,
      COALESCE(t.name, q.product_name) AS task_name
    FROM task_queue q
    LEFT JOIN scm.item_v i
      ON i.item_uuid = q.item_uuid
    LEFT JOIN scm.task t
      USING (task_uuid)
    ORDER BY q.batch_priority, q.sequence_num
  ) r;
END
$$
LANGUAGE 'plpgsql';
