CREATE TABLE scm.work_center (
  work_center_uuid uuid DEFAULT uuid_generate_v4() PRIMARY KEY,
  name             text,
  url              text
);

CREATE TABLE scm.route (
  route_uuid   uuid DEFAULT uuid_generate_v4() PRIMARY KEY,
  product_uuid uuid REFERENCES prd.product (product_uuid) ON DELETE CASCADE,
  name         text
);

CREATE TABLE scm.task (
  task_uuid        uuid DEFAULT uuid_generate_v4() PRIMARY KEY,
  product_uuid     uuid REFERENCES prd.product (product_uuid) ON DELETE SET NULL,
  route_uuid       uuid REFERENCES scm.route (route_uuid) ON DELETE CASCADE,
  name             text,
  description      text,
  data             jsonb,
  sequence_num     integer,
  work_center_uuid uuid REFERENCES scm.work_center (work_center_uuid) ON DELETE SET NULL,
  finished_at      timestamptz,
  created          timestamptz DEFAULT CURRENT_TIMESTAMP,
  modified         timestamptz DEFAULT CURRENT_TIMESTAMP
);

CREATE TABLE scm.batch (
  batch_uuid uuid DEFAULT uuid_generate_v4() PRIMARY KEY,
  priority   integer DEFAULT 0,
  data       jsonb,
  created    timestamptz DEFAULT CURRENT_TIMESTAMP
);

/**
 * A record in scm.task_item represents an instance of a task for a particular
 * item.
 */
CREATE TABLE scm.task_item (
  task_uuid      uuid REFERENCES scm.task (task_uuid) ON DELETE CASCADE,
  item_uuid      uuid REFERENCES scm.item (item_uuid) ON DELETE CASCADE,
  batch_uuid     uuid REFERENCES scm.batch (batch_uuid) ON DELETE SET NULL,
  sorting_num    integer,
  finished_at    timestamptz,
  queue_num      serial,
  PRIMARY KEY (task_uuid, item_uuid)
);

CREATE TABLE scm.product_route (
  product_uuid uuid REFERENCES prd.product (product_uuid),
  route_uuid   uuid REFERENCES scm.route (route_uuid)
);

CREATE OR REPLACE VIEW scm.task_queue_v AS
  SELECT
    ti.task_uuid,
    ti.item_uuid,
    ti.finished_at,
    ti.batch_uuid,
    ti.sorting_num,
    ti.queue_num,
    b.priority AS batch_priority,
    t.route_uuid,
    t.work_center_uuid,
    t.sequence_num,
    p.name AS product_name
  FROM scm.task_item ti
  LEFT JOIN scm.batch b
    ON b.batch_uuid = ti.batch_uuid
  LEFT JOIN scm.task t
    USING (task_uuid)
  LEFT JOIN prd.product_v p
    ON p.product_uuid = t.product_uuid;
